@@ -6,16 +6,16 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
-Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2021 Daniel T. Borelli <daltomi@disroot.org>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
 Copyright 2020      blockparole
 Copyright 2020      Cungsten Tarbide <ctarbide@tuta.io>
-Copyright 2020      daltomi <daltomi@disroot.org>
 Copyright 2020      Hinigatsu <hinigatsu@protonmail.com>
 Copyright 2020      nothub
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
-Copyright 2020      spycapitan <spycapitan@protonmail.com>
 Copyright 2021      c0dev0id <sh+github@codevoid.de>
+Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
+Copyright 2021      Guilherme Janczak <guilherme.janczak@yandex.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -40,6 +40,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "scrot.h"
 #include "options.h"
+#include "slist.h"
 #include <assert.h>
 
 
@@ -53,6 +54,14 @@ static void uninit_x_and_imlib(void)
 }
 
 
+// It assumes that the local variable 'main.c:Imlib_Image image' is in context
+static void apply_filter_if_required(void)
+{
+  if (opt.script != NULL) {
+      imlib_apply_filter(opt.script);
+  }
+}
+
 int
 main(int argc,
      char **argv)
@@ -74,8 +83,8 @@ main(int argc,
   atexit(uninit_x_and_imlib);
 
   if (!opt.output_file) {
-    opt.output_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot.png");
-    opt.thumb_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot-thumb.png");
+    opt.output_file = strdup("%Y-%m-%d-%H%M%S_$wx$h_scrot.png");
+    opt.thumb_file = strdup("%Y-%m-%d-%H%M%S_$wx$h_scrot-thumb.png");
   } else {
     scrot_have_file_extension(opt.output_file, &have_extension);
   }
@@ -102,8 +111,10 @@ main(int argc,
   if (opt.note != NULL)
     scrot_note_draw(image);
 
-  if (!image)
-    gib_eprintf("no image grabbed");
+  if (!image) {
+    fprintf(stderr, "no image grabbed: %s", strerror(errno));
+    exit(EXIT_FAILURE);
+  }
 
   time(&t); /* Get the time directly after the screenshot */
   tm = localtime(&t);
@@ -118,16 +129,20 @@ main(int argc,
   filename_im = im_printf(opt.output_file, tm, NULL, NULL, image);
   scrot_check_if_overwrite_file(&filename_im);
 
-  gib_imlib_save_image_with_error_return(image, filename_im, &err);
-  if (err)
-    gib_eprintf("Saving to file %s failed\n", filename_im);
+  apply_filter_if_required();
+
+  imlib_save_image_with_error_return(filename_im, &err);
+  if (err) {
+    fprintf(stderr, "Saving to file %s failed: %s\n", filename_im, strerror(errno));
+    exit(EXIT_FAILURE);
+  }
   if (opt.thumb)
   {
     int cwidth, cheight;
     int twidth, theight;
 
-    cwidth = gib_imlib_image_get_width(image);
-    cheight = gib_imlib_image_get_height(image);
+    cwidth = imlib_image_get_width();
+    cheight = imlib_image_get_height();
 
     /* Geometry based thumb size */
     if (opt.thumb_width || opt.thumb_height)
@@ -154,11 +169,14 @@ main(int argc,
       theight = cheight * opt.thumb / 100;
     }
 
+    imlib_context_set_anti_alias(1);
     thumbnail =
-      gib_imlib_create_cropped_scaled_image(image, 0, 0, cwidth, cheight,
-                                            twidth, theight, 1);
-    if (thumbnail == NULL)
-      gib_eprintf("Unable to create scaled Image\n");
+      imlib_create_cropped_scaled_image(0, 0, cwidth, cheight,
+                                            twidth, theight);
+    if (thumbnail == NULL) {
+      fprintf(stderr, "Unable to create scaled Image: %s\n", strerror(errno));
+      exit(EXIT_FAILURE);
+    }
     else
     {
       if (opt.note != NULL)
@@ -173,14 +191,19 @@ main(int argc,
 
       filename_thumb = im_printf(opt.thumb_file, tm, NULL, NULL, thumbnail);
       scrot_check_if_overwrite_file(&filename_thumb);
-      gib_imlib_save_image_with_error_return(thumbnail, filename_thumb, &err);
-      if (err)
-        gib_eprintf("Saving thumbnail %s failed\n", filename_thumb);
+
+      apply_filter_if_required();
+
+      imlib_save_image_with_error_return(filename_thumb, &err);
+      if (err) {
+        fprintf(stderr, "Saving thumbnail %s failed: %s\n", filename_thumb, strerror(errno));
+	exit(EXIT_FAILURE);
+      }
     }
   }
   if (opt.exec)
     scrot_exec_app(image, tm, filename_im, filename_thumb);
-  gib_imlib_free_image_and_decache(image);
+  imlib_free_image_and_decache();
 
   return 0;
 }
@@ -305,18 +328,18 @@ scrot_grab_mouse_pointer(const Imlib_Image image,
 {
   XFixesCursorImage *xcim = XFixesGetCursorImage(disp);
 
-  const int width       = xcim->width;
-  const int height      = xcim->height;
-  const int x           = (xcim->x - xcim->xhot) - ix_off;
-  const int y           = (xcim->y - xcim->yhot) - iy_off;
-  DATA32 *pixels        = NULL;
+  const unsigned short width       = xcim->width;
+  const unsigned short height      = xcim->height;
+  const int x                      = (xcim->x - xcim->xhot) - ix_off;
+  const int y                      = (xcim->y - xcim->yhot) - iy_off;
+  DATA32 *pixels                   = NULL;
 
 #ifdef __i386__
   pixels = (DATA32*)xcim->pixels;
 #else
   DATA32 data[width * height * 4];
-  
-  size_t i;
+
+  unsigned int i;
   for (i = 0; i < (width * height); i++)
     ((DATA32*)data)[i] = (DATA32)xcim->pixels[i];
 
@@ -349,7 +372,7 @@ scrot_grab_shot(void)
   if (! opt.silent) XBell(disp, 0);
 
   im =
-    gib_imlib_create_image_from_drawable(root, 0, 0, 0, scr->width,
+    imlib_create_image_from_drawable(0, 0, 0, scr->width,
                                          scr->height, 1);
   if (opt.pointer == 1)
     scrot_grab_mouse_pointer(im, 0, 0);
@@ -391,7 +414,7 @@ scrot_grab_focused(void)
   XGetInputFocus(disp, &target, &ignored);
   if (!scrot_get_geometry(target, &rx, &ry, &rw, &rh)) return NULL;
   scrot_nice_clip(&rx, &ry, &rw, &rh);
-  im = gib_imlib_create_image_from_drawable(root, 0, rx, ry, rw, rh, 1);
+  im = imlib_create_image_from_drawable(0, rx, ry, rw, rh, 1);
   if (opt.pointer == 1)
 	  scrot_grab_mouse_pointer(im, rx, ry);
   return im;
@@ -453,8 +476,39 @@ scrot_sel_and_grab_image(void)
           done = 1;
           break;
         case KeyPress:
-          fprintf(stderr, "Key was pressed, aborting shot\n");
-          done = 2;
+
+          if (!btn_pressed) {
+key_abort_shot:
+              fprintf(stderr, "Key was pressed, aborting shot\n");
+              done = 2;
+              break;
+          }
+
+          KeySym *keysym = NULL;
+          int keycode; /*dummy*/
+
+          keysym = XGetKeyboardMapping(disp, ev.xkey.keycode, 1, &keycode);
+
+          if (keysym == NULL) break;
+
+          switch (*keysym) {
+          case XK_Right:
+              if (++rx > scr->width) rx = scr->width;
+              break;
+          case XK_Left:
+              if (--rx < 0 ) rx = 0;
+              break;
+          case XK_Down:
+              if (++ry > scr->height) ry = scr->height;
+              break;
+          case XK_Up:
+              if (--ry < 0 ) ry = 0;
+              break;
+          default:
+              goto key_abort_shot;
+          }
+          XFree(keysym);
+          scrot_selection_motion_draw(rx, ry, ev.xbutton.x, ev.xbutton.y);
           break;
         case KeyRelease:
           /* ignore */
@@ -482,7 +536,7 @@ scrot_sel_and_grab_image(void)
 
   XUngrabKeyboard(disp, CurrentTime);
 
-  bool const isAreaSelect = (bool)(scrot_selection_get_rect().w > 5);
+  bool const isAreaSelect = (bool)(scrot_selection_get_rect()->w > 5);
 
   scrot_selection_destroy();
 
@@ -493,6 +547,14 @@ scrot_sel_and_grab_image(void)
       rw = ev.xbutton.x - rx;
       rh = ev.xbutton.y - ry;
 
+      if ((ev.xbutton.x + 1) == WidthOfScreen(scr)) {
+        ++rw;
+      }
+
+      if ((ev.xbutton.y + 1) == HeightOfScreen(scr)) {
+        ++rh;
+      }
+
       if (rw < 0) {
         rx += rw;
         rw = 0 - rw;
@@ -502,8 +564,11 @@ scrot_sel_and_grab_image(void)
         rh = 0 - rh;
       }
 
-      // Not record pointer if there is a selection area because it is busy on that
-      opt.pointer = 0;
+      // Not record pointer if there is a selection area because it is busy on that,
+      // unless the delay option is used
+      if (opt.delay == 0) {
+          opt.pointer = 0;
+      }
 
     } else {
       /* else it's a window click */
@@ -512,7 +577,7 @@ scrot_sel_and_grab_image(void)
     scrot_nice_clip(&rx, &ry, &rw, &rh);
 
     if (! opt.silent) XBell(disp, 0);
-    im = gib_imlib_create_image_from_drawable(root, 0, rx, ry, rw, rh, 1);
+    im = imlib_create_image_from_drawable(0, rx, ry, rw, rh, 1);
 
     if (opt.pointer == 1)
        scrot_grab_mouse_pointer(im, rx, ry);
@@ -528,7 +593,7 @@ scrot_grab_autoselect(void)
 
   scrot_do_delay();
   scrot_nice_clip(&rx, &ry, &rw, &rh);
-  im = gib_imlib_create_image_from_drawable(root, 0, rx, ry, rw, rh, 1);
+  im = imlib_create_image_from_drawable(0, rx, ry, rw, rh, 1);
   if (opt.pointer == 1)
 	  scrot_grab_mouse_pointer(im, rx, ry);
   return im;
@@ -555,6 +620,13 @@ scrot_nice_clip(int *rx,
     *rh = scr->height - *ry;
 }
 
+static Bool scrot_xevent_visibility(Display *dpy, XEvent *ev, XPointer arg)
+{
+    (void) dpy; // unused
+    Window *win = (Window*)arg;
+    return (ev->xvisibility.window == *win);
+}
+
 /* get geometry of window and use that */
 int
 scrot_get_geometry(Window target,
@@ -565,7 +637,7 @@ scrot_get_geometry(Window target,
 {
   Window child;
   XWindowAttributes attr;
-  int stat;
+  int stat, frames = 0;
 
   /* get windowmanager frame of window */
   if (target != root) {
@@ -576,20 +648,36 @@ scrot_get_geometry(Window target,
     status = XGetGeometry(disp, target, &root, &x, &x, &d, &d, &d, &d);
     if (status != 0) {
       Window rt, *children, parent;
+      XEvent ev;
 
       for (;;) {
-	/* Find window manager frame. */
-	status = XQueryTree(disp, target, &rt, &parent, &children, &d);
-	if (status && (children != None))
-	  XFree((char *) children);
-	if (!status || (parent == None) || (parent == rt))
-	  break;
-	target = parent;
+        /* Find window manager frame. */
+        status = XQueryTree(disp, target, &rt, &parent, &children, &d);
+        if (status && (children != None)) {
+          XFree((char *) children);
+        }
+        if (!status || (parent == None) || (parent == rt)) {
+            break;
+        }
+        target = parent;
+        ++frames;
       }
       /* Get client window. */
-      if (!opt.border)
-	target = scrot_get_client_window(disp, target);
+      if (!opt.border) {
+        target = scrot_get_client_window(disp, target);
+      }
       XRaiseWindow(disp, target);
+
+      /* Give the WM time to update the hidden area of the window.
+         Some windows never send the event, a time limit is placed.
+      */
+      XSelectInput(disp, target, FocusChangeMask);
+      for(short i = 0; i < 30; ++i) {
+        if (XCheckIfEvent(disp, &ev, &scrot_xevent_visibility, (XPointer)&target) == True) {
+            break;
+        }
+        usleep(2000);
+      }
     }
   }
   stat = XGetWindowAttributes(disp, target, &attr);
@@ -598,6 +686,14 @@ scrot_get_geometry(Window target,
   *rw = attr.width;
   *rh = attr.height;
   XTranslateCoordinates(disp, target, root, 0, 0, rx, ry, &child);
+
+  /* Special case when the TWM emulates the border directly on the window. */
+  if (opt.border == 1 && frames < 2 && attr.border_width > 0) {
+      *rw += attr.border_width * 2;
+      *rh += attr.border_width * 2;
+      *rx -= attr.border_width;
+      *ry -= attr.border_width;
+  }
   return 1;
 }
 
@@ -654,6 +750,7 @@ im_printf(char *str, struct tm *tm,
     exit(EXIT_FAILURE);
   }
 
+  imlib_context_set_image(im);
   for (c = strf; *c != '\0'; c++) {
     if (*c == '$') {
       c++;
@@ -666,7 +763,7 @@ im_printf(char *str, struct tm *tm,
           if (filename_im)
             strcat(ret, filename_im);
           break;
-        case 'm': /* t was allready taken, so m as in mini */
+        case 'm': /* t was already taken, so m as in mini */
           if (filename_thumb)
             strcat(ret, filename_thumb);
           break;
@@ -680,11 +777,11 @@ im_printf(char *str, struct tm *tm,
           }
           break;
         case 'w':
-          snprintf(buf, sizeof(buf), "%d", gib_imlib_image_get_width(im));
+          snprintf(buf, sizeof(buf), "%d", imlib_image_get_width());
           strcat(ret, buf);
           break;
         case 'h':
-          snprintf(buf, sizeof(buf), "%d", gib_imlib_image_get_height(im));
+          snprintf(buf, sizeof(buf), "%d", imlib_image_get_height());
           strcat(ret, buf);
           break;
         case 's':
@@ -701,14 +798,14 @@ im_printf(char *str, struct tm *tm,
           break;
         case 'p':
           snprintf(buf, sizeof(buf), "%d",
-                   gib_imlib_image_get_width(im) *
-                   gib_imlib_image_get_height(im));
+                   imlib_image_get_width() *
+                   imlib_image_get_height());
           strcat(ret, buf);
           break;
         case 't':
-          tmp = gib_imlib_image_format(im);
+          tmp = imlib_image_format();
           if (tmp) {
-            strcat(ret, gib_imlib_image_format(im));
+            strcat(ret, tmp);
           }
           break;
         case '$':
@@ -735,7 +832,7 @@ im_printf(char *str, struct tm *tm,
       ret[len + 1] = '\0';
     }
   }
-  return gib_estrdup(ret);
+  return strdup(ret);
 }
 
 Window
@@ -813,10 +910,12 @@ scrot_grab_stack_windows(void)
     Bool delete      = False;
     int actual_format_return;
     Atom actual_type_return;
-    gib_list *list_images   = NULL;
+    Scrot_Imlib_List *list_images   = NULL;
     Imlib_Image im          = NULL;
     XImage *ximage          = NULL;
     XWindowAttributes attr;
+    unsigned long i = 0;
+
 
 #define EWMH_CLIENT_LIST "_NET_CLIENT_LIST" // spec EWMH
 
@@ -834,7 +933,6 @@ scrot_grab_stack_windows(void)
 
     XCompositeRedirectSubwindows(disp, root, CompositeRedirectAutomatic);
 
-    int i;
     for (i = 0; i < nitems_return; i++) {
 
         Window win = *((Window*)prop_return + i);
@@ -864,7 +962,7 @@ scrot_grab_stack_windows(void)
 
         XFree(ximage);
 
-        list_images = gib_list_add_end(list_images, im);
+        list_images = append_to_scrot_imlib(list_images, im);
     }
 
     return stalk_image_concat(list_images);
@@ -877,14 +975,14 @@ scrot_grab_shot_multi(void)
   int i;
   char *dispstr, *subdisp;
   char newdisp[255];
-  gib_list *images = NULL;
+  Scrot_Imlib_List *images = NULL;
   Imlib_Image ret = NULL;
 
   screens = ScreenCount(disp);
   if (screens < 2)
     return scrot_grab_shot();
 
-  subdisp = gib_estrdup(DisplayString(disp));
+  subdisp = strdup(DisplayString(disp));
 
   for (i = 0; i < screens; i++) {
     dispstr = strchr(subdisp, ':');
@@ -896,9 +994,9 @@ scrot_grab_shot_multi(void)
     snprintf(newdisp, sizeof(newdisp), "%s.%d", subdisp, i);
     init_x_and_imlib(newdisp, i);
     ret =
-      gib_imlib_create_image_from_drawable(root, 0, 0, 0, scr->width,
+      imlib_create_image_from_drawable(0, 0, 0, scr->width,
                                            scr->height, 1);
-    images = gib_list_add_end(images, ret);
+    images = append_to_scrot_imlib(images, ret);
   }
   free(subdisp);
 
@@ -908,39 +1006,48 @@ scrot_grab_shot_multi(void)
 }
 
 Imlib_Image
-stalk_image_concat(gib_list * images)
+stalk_image_concat(Scrot_Imlib_List * images)
 {
   int tot_w = 0, max_h = 0, w, h;
   int x = 0;
-  gib_list *l, *item;
+  Scrot_Imlib_List *l, *item;
   Imlib_Image ret, im;
 
-  if (gib_list_length(images) == 0)
+  if (is_scrot_imlib_list_empty(images))
     return NULL;
 
   l = images;
   while (l) {
     im = (Imlib_Image) l->data;
-    h = gib_imlib_image_get_height(im);
-    w = gib_imlib_image_get_width(im);
+    imlib_context_set_image(im);
+    h = imlib_image_get_height();
+    w = imlib_image_get_width();
     if (h > max_h)
       max_h = h;
     tot_w += w;
     l = l->next;
   }
   ret = imlib_create_image(tot_w, max_h);
-  gib_imlib_image_fill_rectangle(ret, 0, 0, tot_w, max_h, 255, 0, 0, 0);
+  imlib_context_set_image(ret);
+  imlib_context_set_color(255, 0, 0, 0);
+  imlib_image_fill_rectangle(0, 0, tot_w, max_h);
   l = images;
   while (l) {
     im = (Imlib_Image) l->data;
     item = l;
     l = l->next;
-    h = gib_imlib_image_get_height(im);
-    w = gib_imlib_image_get_width(im);
-    gib_imlib_blend_image_onto_image(ret, im, 0, 0, 0, w, h, x, 0, w, h, 1, 0,
-                                     0);
+    imlib_context_set_image(im);
+    h = imlib_image_get_height();
+    w = imlib_image_get_width();
+    imlib_context_set_image(ret);
+    imlib_context_set_anti_alias(0);
+    imlib_context_set_dither(1);
+    imlib_context_set_blend(0);
+    imlib_context_set_angle(0);
+    imlib_blend_image_onto_image(im, 0, 0, 0, w, h, x, 0, w, h);
     x += w;
-    gib_imlib_free_image_and_decache(im);
+    imlib_context_set_image(im);
+    imlib_free_image_and_decache();
     free(item);
   }
   return ret;
