@@ -1,7 +1,6 @@
 /* note.c
 
-Copyright 2019 Daniel T. Borelli <danieltborelli@gmail.com>
-Copyright 2020 spycapitan <spycapitan@protonmail.com>
+Copyright 2019-2020 Daniel T. Borelli <daltomi@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -31,7 +30,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "scrot.h"
 
-
 scrotnote note;
 
 static Imlib_Font imfont = NULL;
@@ -158,10 +156,10 @@ malformed:
            }
 
            switch(++nclrs) {
-           case 1 : note.color.r = color;
-           case 2 : note.color.g = color;
-           case 3 : note.color.b = color;
-           case 4 : note.color.a = color;
+           case 1 : note.color.r = color; break;
+           case 2 : note.color.g = color; break;
+           case 3 : note.color.b = color; break;
+           case 4 : note.color.a = color; break;
            }
 
            c = strtok_r(NULL, ",", &saveptr);
