@@ -9,11 +9,10 @@ Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel Lublin <daniel@lublin.se>
-Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2021 Daniel T. Borelli <daltomi@disroot.org>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
-Copyright 2020      daltomi <daltomi@disroot.org>-
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
-Copyright 2020      spycapitan <spycapitan@protonmail.com>
+Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -218,7 +217,7 @@ static void options_parse_window_class_name(const char* window_class_name)
 static void
 scrot_parse_option_array(int argc, char **argv)
 {
-   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:zn:l:D:kC:";
+   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:zn:l:D:kC:S:";
 
    static struct option lopts[] = {
       /* actions */
@@ -246,6 +245,7 @@ scrot_parse_option_array(int argc, char **argv)
       {"note", required_argument, 0, 'n'},
       {"line", required_argument, 0, 'l'},
       {"class", required_argument, 0, 'C'},
+      {"script", required_argument, 0, 'S'},
       {0, 0, 0, 0}
    };
    int optch = 0, cmdx = 0;
@@ -271,7 +271,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.delay = options_parse_required_number(optarg);
            break;
         case 'e':
-           opt.exec = gib_estrdup(optarg);
+           opt.exec = strdup(optarg);
            break;
         case 'm':
            opt.multidisp = 1;
@@ -324,6 +324,9 @@ scrot_parse_option_array(int argc, char **argv)
         case 'C':
             options_parse_window_class_name(optarg);
         break;
+        case 'S':
+           opt.script = strdup(optarg);
+        break;
         case '?':
            exit(EXIT_FAILURE);
         default:
@@ -332,27 +335,24 @@ scrot_parse_option_array(int argc, char **argv)
    }
 
    /* Now the leftovers, which must be files */
-   if (optind < argc)
+   while (optind < argc)
    {
-      while (optind < argc)
+      /* If recursive is NOT set, but the only argument is a directory
+         name, we grab all the files in there, but not subdirs */
+      if (!opt.output_file)
       {
-         /* If recursive is NOT set, but the only argument is a directory
-            name, we grab all the files in there, but not subdirs */
-         if (!opt.output_file)
-         {
-            opt.output_file = argv[optind++];
+         opt.output_file = argv[optind++];
 
-            if ( strlen(opt.output_file) > 256 ) {
-               printf("output filename too long.\n");
-               exit(EXIT_FAILURE);
-            }
-
-            if (opt.thumb)
-               opt.thumb_file = name_thumbnail(opt.output_file);
+         if ( strlen(opt.output_file) > 256 ) {
+            printf("output filename too long.\n");
+            exit(EXIT_FAILURE);
          }
-         else
-            gib_weprintf("unrecognised option %s\n", argv[optind++]);
+
+         if (opt.thumb)
+            opt.thumb_file = name_thumbnail(opt.output_file);
       }
+      else
+         fprintf(stderr, "unrecognised option %s\n", argv[optind++]);
    }
 
    /* So that we can safely be called again */
@@ -368,7 +368,11 @@ name_thumbnail(char *name)
    size_t diff = 0;
 
    length = strlen(name) + 7;
-   new_title = gib_emalloc(length);
+   new_title = malloc(length);
+   if (! new_title) {
+     fprintf(stderr, "Unable to allocate thumbnail: %s", strerror(errno));
+     exit(EXIT_FAILURE);
+   }
 
    dot_pos = strrchr(name, '.');
    if (dot_pos)
@@ -422,6 +426,9 @@ options_parse_autoselect(char *optarg)
 void
 options_parse_display(char *optarg)
 {
+#if SCROT_HAVE_STRNDUP
+    opt.display = strndup(optarg, 256);
+#else
    size_t length = 0;
    char *new_display;
 
@@ -429,9 +436,14 @@ options_parse_display(char *optarg)
    if (length > 256) {
      length = 256;
    }
-   new_display = gib_emalloc(length);
+   new_display = malloc(length);
+   if (! new_display) {
+     fprintf(stderr, "Unable to allocate display: %s", strerror(errno));
+     exit(EXIT_FAILURE);
+   }
    strncpy(new_display, optarg, length);
    opt.display=new_display;
+#endif
 }
 
 void
@@ -472,7 +484,7 @@ options_parse_thumbnail(char *optarg)
 
 void options_parse_note(char *optarg)
 {
-   opt.note = gib_estrdup(optarg);
+   opt.note = strdup(optarg);
 
    if (opt.note == NULL) return;
 
@@ -526,6 +538,7 @@ show_usage(void)
            "  -D, --display             Set DISPLAY target other than current\n"
            "  -a, --autoselect          non-interactively choose a rectangle of x,y,w,h\n"
            "  -b, --border              When selecting a window, grab wm border too\n"
+           "                            Use with --select to raise the focus of the window.\n"
            "  -c, --count               show a countdown before taking the shot\n"
            "  -d, --delay NUM           wait NUM seconds before taking a shot\n"
            "  -e, --exec APP            run APP on the resulting screenshot\n"
@@ -536,7 +549,7 @@ show_usage(void)
            "  -m, --multidisp           For multiple heads, grab shot from each\n"
            "                            and join them together.\n"
            "  -s, --select              interactively choose a window or rectangle\n"
-           "                            with the mouse\n"
+           "                            with the mouse (use the arrow keys to resize)\n"
            "  -u, --focused             use the currently focused window\n"
            "  -t, --thumb NUM           generate thumbnail too. NUM is the percentage\n"
            "                            of the original size for the thumbnail to be,\n"
@@ -549,9 +562,10 @@ show_usage(void)
            "                            See SELECTION STYLE\n"
            "  -n, --note                Draw a text note.\n"
            "                            See NOTE FORMAT\n"
-           "  -k, --stack               Capture stack/overlaped windows and join them together.\n"
+           "  -k, --stack               Capture stack/overlapped windows and join them together.\n"
            "                            A running Composite Manager is needed.\n"
            "  -C,  --class NAME         Window class name. Associative with options: -k\n"
+           "  -S,  --script CMD         Imlib2 script commands\n"
            "\n" "  SPECIAL STRINGS\n"
            "  Both the --exec and filename parameters can take format specifiers\n"
            "  that are expanded by " SCROT_PACKAGE " when encountered.\n"
@@ -586,7 +600,7 @@ show_usage(void)
            "  Mode 'edge' ignore    : style, --freeze\n"
            "  Mode 'classic' ignore : opacity\n\n"
            "  The 'opacity' specifier is only effective if a Composite Manager is running.\n\n"
-           "  For the color you can use a name or a hexdecimal value.\n"
+           "  For the color you can use a name or a hexadecimal value.\n"
            "                  color=\"red\" or color=\"#ff0000\"\n"
            "  Example:\n" "          " SCROT_PACKAGE
            " --line style=dash,width=3,color=\"red\" --select\n\n"
