@@ -4,11 +4,11 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
                                   gilbertt@linuxbrit.co.uk,
                                   scrot_sucks@linuxbrit.co.uk>
 Copyright 2009      James Cameron <quozl@us.netrek.org>
-Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
-Copyright 2020      daltomi <daltomi@disroot.org>
+Copyright 2019-2020 Daniel T. Borelli <daltomi@disroot.org>
 Copyright 2020      Jeroen Roovers <jer@gentoo.org>
 Copyright 2020      Hinigatsu <hinigatsu@protonmail.com>
-Copyright 2020      spycapitan <spycapitan@protonmail.com>
+Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
+Copyright 2021      Guilherme Janczak <guilherme.janczak@yandex.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -60,16 +60,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <time.h>
 #include <signal.h>
 #include <sys/wait.h>
-#include <giblib/giblib.h>
+#include <Imlib2.h>
 #include <stdbool.h>
-
+#include <getopt.h>
 
 #include "scrot_config.h"
 #include "structs.h"
-#include "getopt.h"
-#include "debug.h"
 #include "note.h"
 #include "scrot_selection.h"
+#include "slist.h"
 
 #ifndef __GNUC__
 # define __attribute__(x)
@@ -102,7 +101,7 @@ char *im_printf(char *str, struct tm *tm,
                 Imlib_Image im);
 Imlib_Image scrot_grab_shot_multi(void);
 Imlib_Image scrot_grab_stack_windows(void);
-Imlib_Image stalk_image_concat(gib_list *images);
+Imlib_Image stalk_image_concat(Scrot_Imlib_List *images);
 
 void scrot_grab_mouse_pointer(const Imlib_Image image,
 		const int ix_off, const int iy_off);
