@@ -1,7 +1,6 @@
 /* scrot_selection_classic.c
 
-Copyright 2020  daltomi <daltomi@disroot.org>
-Copyright 2020  Daniel T. Borelli <daltomi@disroot.org>
+Copyright 2020-2021 Daniel T. Borelli <daltomi@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -86,7 +85,7 @@ void selection_classic_create(void)
 }
 
 
-void selection_classic_destroy()
+void selection_classic_destroy(void)
 {
     struct selection_t const *const sel = *selection_get();
     struct selection_classic_t* pc = sel->classic;
@@ -103,7 +102,7 @@ void selection_classic_destroy()
 }
 
 
-void selection_classic_draw()
+void selection_classic_draw(void)
 {
     struct selection_t const *const sel = *selection_get();
     struct selection_classic_t const *const pc = sel->classic;
